from dataclasses import dataclass
from collections.abc import Sequence
from typing import Collection, Iterable

from simplest.structures.mlst import Allele
from simplest.structures.sequences import NamedString

class SequenceTypeDefinitions:
    def __init__(self, loci_names: Iterable[str], sequence_types: Iterable[tuple[dict[str, str], str]]):
        self._locis = sorted(loci_names)
        self._sequence_type_map: dict[tuple, str] = dict()

        for loci_alleles, st in sequence_types:
            if st in self._sequence_type_map:
                # TODO raise exception for duplicate sequence type definitions
                pass
            self._sequence_type_map[self.order_profile_by_loci_order(loci_alleles)] = st

    def get_ordered_locis(self):
        return tuple(self._locis)

    def order_profile_by_loci_order(self, alleles: dict[str, str]) -> tuple[str]:
        ordered_alleles = []
        for allele in self._locis:
            ordered_alleles.append(alleles[allele])
        # TODO raise exception if length of ordered alleles is not the same as the number of sequence type alleles
        return tuple(ordered_alleles)

    def get_sequence_type_from_alleles(self, alleles: Iterable[Allele]):
        allele_map: dict[str, Allele] = dict()
        ordered_alleles: list[str] = list()
        for allele in alleles:
            if allele.allele_locus in allele_map:
                # TODO raise exception
                pass
            allele_map[allele.allele_locus] = allele
        for locus in self.get_ordered_locis():
            if locus not in allele_map:
                # TODO raise exception for missing alleles
                pass
            ordered_alleles.append(allele_map[locus].allele_variant)
        if tuple(ordered_alleles) not in self._sequence_type_map:
            return "unknown"
        return self._sequence_type_map[tuple(ordered_alleles)]
    